
/* 	This file implements Labeled Tree 
	defined by Susan Davidson in University of Pennsylvania
	,published in SIGMOD 96

	9 operators defined through, this file and LTlabel
*/


import java.lang.*;
import java.util.*;

public class LTtree {

	LTmarker marker;
	Vector set;

	public LTtree(){
	//empty tree
	//correspond to the operator, {}
		set = new Vector();
	}

	public LTtree(LTmarker m){
	//empty tree with marker
	//correspond to the operator, X
		set = new Vector();
		marker = m;
	}

	public LTtree( LTlabel label, LTtree tree ){
	//don't create new objects, attach root of tree to label
	//correspond to the operator, l => t

		set = new Vector();
		set.addElement( new LTpair(label, tree) );
	}

	public LTtree( LTtree t1, LTtree t2 ){
	//don't create new nodes, just merge two trees
	//correspond to the operator, t1 U t2

		set = new Vector();
		int i = 0;
		for( i = 0 ; i < t1.getSet().size() ; i++ ){
			set.addElement( t1.getSet().elementAt( i ) );
		}
		for( i = 0 ; i < t2.getSet().size() ; i++ ){
			set.addElement( t2.getSet().elementAt( i ) );
		}
	}

	public boolean isEmpty(){
	//check if tree is empty or not
	//correspond to the operator, isempty(t)

		if( set.size() <= 0 ) return true;
		else return false;
	}

	public void append( Vector trees ){
	//attach X-indexed LTs to each markers
	//trees:X-indexed family of Labeled Trees
	//correspond to the operator, t @x s

		if( trees == null ) return;

		for( int i=0; i < set.size(); i++ ){
			LTpair tmp = (LTpair)set.elementAt(i);

			//recursively traverse LT , find Marker and
			// replace the Marker with Marker-indexed LT
			if(	!tmp.getSubtree().isEmpty() ) 
				tmp.getSubtree().append( trees );

			LTlabel label_tmp = tmp.getLabel();
			if( label_tmp.isMarker() ){

				for( int index = 0; index < trees.size(); index++ ){
				// traverse the X-indexed family and
				// assign each tree to corresponding Marker

					LTtree ptrTree = (LTtree) trees.elementAt( index );
					LTmarker ptrMarker = ptrTree.getMarker();
					//working 
					if( ptrMarker == label_tmp.getMarker() ){
					//replace the marker with the tree
						label_tmp.setLabel("null", LTlabel.EMPTY);
						tmp.setTree(ptrTree);
					}
				}
			}
		}

	}

	public LTmarker getMarker(){ return marker; }
	public void setMarker( LTmarker m ){ marker = m; }

	public MTree makeMTree(){
	// libero.UI.TreeUI draw MTree on window
	// so, this function provides graphical view of LT

		MTree mtree = new MTree("LT");
		Node root = mtree.getRoot();

		for( int i=0; i < set.size() ; i++ ){
			root.addChild( makeNode( (LTpair )set.elementAt(i)) );
		}
		return mtree; 
	}

	private Node makeNode( LTpair t){
	//reculsively used in makeMTree() to make LT to mtree
		Node n = new Node( t.getLabel().getString() );

		for( int i=0; i < t.getSubtree().getSet().size() ; i++ ){
			n.addChild( makeNode (
				(LTpair) t.getSubtree().getSet().elementAt(i)
				)
			);
		}
		return n;
	}

	private Vector getSet(){
		return set;
	}
}

class LTpair extends Object{
// a pair of label, and LT

	LTlabel label;
	LTtree	tree;
	LTpair( LTlabel l, LTtree t){
		label = l;
		tree	= t ;
	}
	void setTree(LTtree t){ tree=t; }
	LTlabel getLabel(){ return label; }
	LTtree getSubtree(){ return tree; }
}
