
import java.lang.*;
import java.util.*;
import java.io.*;

/**
* Node is needed to create Mtree type.
* and maintain tree type.
* each Node has pointers of children 
*
* @author	Joon-Myung Kang
* @version	0.5
* @since	1999/5/22
*/

public class Node extends Object
{
	/**
	* information of this Node
	*/
	String str ;
	/**
	* pointers of children
	*/
	Vector child ;

	/**
	* Constructor.
	* 
	* @param s node string
	* @return none
	*/
	public Node( String s )
	{
		str = s;
		child =  new Vector(); 
	}

	/**
	* attach n to this node
	*
	* @param n Node to attach
	* @return Node
	*/
	public Node addChild( Node n )
	{
		child.addElement( n );
		return n;
	}

	/**
	* get the label of node.
	* 
	* @param none
	* @return label of node
	*/
	public String getLabel()
	{
		return str;
	}

	/**
	* change the label of node.
	*
	* @param s label
	* @return none
	*/
	public	void	changeLabel(String s)
	{
		str = s;
	}

	/**
	* return size of the children
	*
	* @param none
	* @return size of the children
	*/
	public int childSize()
	{
		return child.size();
	}

	/**
	* return ith node
	*
	* @param i ith node
	* @return Node of ith
	*/
	public Node getChildAt(int i)
	{
		return ( Node )child.elementAt(i);
	}

	/**
	* copy the whole Node including subtree.
	*
	* @param none
	* @return Node
	*/
	public	Node	copyTree()
	{
		Node	copyNode = new Node(str);
		for(int i=0;i<child.size();i++)
		{
			copyNode.addChild(((Node)child.elementAt(i)).copyTree());
		}
		return copyNode;
	}

	/**
	* <pre>
	* -inputString : query string
	* 	ex) "*.CENTER.BODY.*.TR"
	* -result : new query string after removing first label
	* 	ex) "CENTER.BODY.*.TR" in case of "*.CENTER.BODY.*.TR"
	* </pre>
	*
	* @param inputString query input string
	* @return query string
	*/
	public	String	getStringAfterLookup(String inputString)
	{
		inputString.trim();	//remove white space 

		if(inputString.length()==0) return new String("");
		int	index = inputString.indexOf(".");
		if(index == -1) return new String("");
		return	inputString.substring(index+1);
	}

	/**
	* <pre>
	* - inputString : query string
	* 	ex) "*.CENTER.BODY.*.TR"
	* - result : first label of the inputString
	*	ex) "*" in case of "*.CENTER.BODY.*.TR"
	* </pre>
	*
	* @param inputString query input string
	* @return query string
	*/
	public	String	lookupNextLabel(String inputString)
	{
		int firstIndexOfLabel = 0;	//starting index of label
		int lastIndexOfLabel = 0;	//ending index of label
		//	ex> when inputString is "html.body.*.TR"
		//		index of 'h' is firstIndexOfLabel
		//		index of 'l' is lastIndexOfLabel

	// for debugging
		//System.out.println("\n inputString : \""+inputString+"\"");

		if(inputString.length()==0) return new String("");
		while(Character.isWhitespace(inputString.charAt(firstIndexOfLabel)))
		//ignore white space
		{
			firstIndexOfLabel++;
			if(inputString.length()==firstIndexOfLabel) return new String("");
		}
		lastIndexOfLabel = firstIndexOfLabel;
		while(inputString.length()>lastIndexOfLabel)
		//find end of label
		//condition	: first meets white space or
		//		: first meets '.'
		{
			if((Character.isWhitespace(inputString.charAt(lastIndexOfLabel)))||
			   (inputString.charAt(lastIndexOfLabel)=='.'))
			{
				break;
			}
				lastIndexOfLabel++;
		}
		lastIndexOfLabel--;

	// for debugging
		//System.out.println("firstIndexOfLabel : "+Integer.toString(firstIndexOfLabel));
		//System.out.println("lastIndexOfLabel : "+Integer.toString(lastIndexOfLabel));
		return	inputString.substring(firstIndexOfLabel,lastIndexOfLabel+1);
	}

	/**
	* query string and path.
	* 
	* @param inputString query string
	* @param path full path of Node
	* @return result of query
	*/
	public	Node	query(String inputString,String path)
	{
		Node	result;		// result Node
		Node	childResult;	// result Node of children

		String	label = lookupNextLabel(inputString);	
		//label := the first label of inputString
		inputString = getStringAfterLookup(inputString);
		String	nextLabel = lookupNextLabel(inputString);

		if(label.equalsIgnoreCase(str))
		//when label equals str
		//*str is member of Node class
		{
			if(nextLabel.length() == 0)
			//when label is a end label of inputString
			//means inputString is just "label"
			{
			//for debugging
				System.out.println("MATCH!!");

				result = new Node("result");
				Node	hitNode = copyTree();	//copy this Node
				hitNode.changeLabel(new String(path+"."+str));	//write full path to Node
				result.addChild(hitNode);	//attach hitNode to result
			}
			else
			if(nextLabel.equalsIgnoreCase("*"))
			//when "*" follows label
			//means inputString is like "label.*" of "label.*.something"
			{
				nextLabel = lookupNextLabel(inputString);	//find next label of label
				inputString = getStringAfterLookup(inputString);	// update inputString
				String	nextNextLabel = lookupNextLabel(inputString);	//find next label of nextLabel
				if(nextNextLabel.length()==0)
				//when nextNextLabel is the end of inputString
				{
					result = query("*",path);
				}
				else
				{
					result = new Node("result");
					for(int i=0; i<childSize(); i++)
					//querying recursively and attach the result
					{
						childResult = getChildAt(i).query(new String("*."+inputString),path+"."+str);
						for(int j=0; j<childResult.childSize(); j++)
						{
							result.addChild(childResult.getChildAt(j));
						}
					}
				}
			}
			else
			//in case of "label.someLabel." 
			{
				result = new Node("result");
				for(int i=0;i<childSize();i++)
				//querying recursively and attach the result
				{
					childResult = getChildAt(i).query(new String(inputString),path+"."+str);
					for(int j=0; j<childResult.childSize(); j++)
					{
						result.addChild(childResult.getChildAt(j));
					}
				}
			}
		}
		else 
		if(label.equalsIgnoreCase("*"))
		//when label equals "*"
		{
			if(nextLabel.length() == 0)
			//when label is the end of inputString
			{
				result = query(new String(str),path);
				for(int i=0; i<childSize(); i++)
				{
					childResult = getChildAt(i).query("*",path+"."+str);
					for(int j=0; j<childResult.childSize(); j++)
					{
						result.addChild(childResult.getChildAt(j));
					}
				}
			}
			else
			//when another label follows label
			{
				result = new Node("result");
				childResult = query(new String(inputString),path);
				for(int i=0;i<childResult.childSize();i++)
				{
					result.addChild(childResult.getChildAt(i));
				}
				for(int i=0; i<childSize(); i++)
				{
					childResult = getChildAt(i).query(new String("*."+inputString),path+"."+str);
					for(int j=0; j<childResult.childSize(); j++)
					{
						result.addChild(childResult.getChildAt(j));
					}
				}
			}
		}
		else
		//when label does not match
		{
			//System.out.println("No Match!!!");
			result = new Node("no result");
		}
		return	result;
	}

	/** 
	* print tree of node
	*
	* @param height height of tree
	* @return none
	*/
	public	void	printTree(int height)
	// this method is for testing program
	{
		String	s="";

		for(int i=0;i<height;i++)
		{
			s=s+"\t";
		}
		s=s+"<"+str+">";
		System.out.println(s);

		for(int i=0;i<child.size();i++)
		{
			((Node)	child.elementAt(i)).printTree(height+1);
		}
	}
	
	/**
	* make tree to file
	*
	* @param stream dataoutput stream
	* @param height height of tree
	* @return none
	*/
	public	void	makeTreeFile(DataOutputStream stream,int height)
	// this method is for testing program
	// out the result of parsing to the file
	{
		String	s="";

		for(int i=0;i<height;i++)
		{
			s=s+"\t";
		}
		s=s+"<"+str+">\n";
		try
		{
			//stream.writeUTF(s);
			stream.writeBytes(s);
		}
		catch(IOException io)
		{
			System.err.println("Error occurred during writing to file\n");
			System.exit(1);
		}
		for(int i=0;i<child.size();i++)
		{
			((Node)	child.elementAt(i)).makeTreeFile(stream,height+1);
		}
	}
}

// end of file
