/** **********************************************************************
 * Console.java
 * A basic graphical console for Jess
 *
 * $Id$
 * (C) 1998 E.J. Friedman-Hill and the Sandia Corporation
 ********************************************************************** */

package jess;

import java.awt.*;
import java.io.*;


public class Console extends Frame
{
  ConsoleDisplay m_display;
  String m_batchfile;

  Console(String title, String[] args)
  {
    super(title);
    m_display = new ConsoleDisplay(null);
    
    // File to batch later
    if (args.length > 0)      
      m_batchfile = args[0];

    add("Center",m_display.panel());
    Button b;
    add("South", b = new Button("Quit"));
    validate();
    resize(500,300);
    show();
  }

  public boolean action(Event e, Object obj)
  {
    if (obj.equals("Quit"))
      {        
        dispose();
        System.exit(0);
        return true;
      }
    return false;
  }

  private void execute()
  {
    Rete r = new Rete(m_display);
    Jesp j = new Jesp(m_display.stdin(), r);

    // Load in optional packages, but don't fail if any are missing.
    String [] packages = { "jess.StringFunctions",
                           "jess.PredFunctions",
                           "jess.MultiFunctions", 
                           "jess.MiscFunctions",
                           "jess.MathFunctions",
                           "jess.BagFunctions",
                           "jess.reflect.ReflectFunctions",
                           "jess.view.ViewFunctions" };
    
    for (int i=0; i< packages.length; i++)
      {
        try
          {
            r.addUserpackage((Userpackage)
                                  Class.forName(packages[i]).newInstance());
          }
        catch (Throwable t) { /* Optional package not present */ }
      }
    
    try
      {
        r.executeCommand("(printout t \"Jess, the Java Expert System " +
                              "Shell\" crlf)");
        r.executeCommand("(printout t \"Copyright (C) 1998 E.J. " +
                              "Friedman-Hill and  the Sandia Corporation\"" +
                              " crlf)");
        
        r.executeCommand("(printout t (jess-version-string) crlf crlf)");
        r.executeCommand("(printout t \"Jess output appears in this " +
                              "window.\" crlf)");
        r.executeCommand("(printout t \"Type your input in the " +
                              "one-line input box below.\" crlf)");


        m_display.setFocus();
        if (m_batchfile != null)
          r.executeCommand("(batch \"" + m_batchfile +"\")");

      }
    catch (ReteException re)
      {
        m_display.stdout().println("ERROR DURING SETUP: " + re.toString());
      }

    // Now simply loop and run commands.
    do 
      {
        try
          {
            m_display.setFocus();
            j.parse(true);
          }
        catch (ReteException re) 
          {
            re.printStackTrace(r.errStream());
          }
        catch (ArrayIndexOutOfBoundsException ai) 
          {
            r.errStream().println("Wrong # of args in funcall?");
            ai.printStackTrace(r.errStream());
          }
        catch (NullPointerException npe) 
          {
            r.errStream().println("Wrong # of args in funcall?");
            npe.printStackTrace(r.errStream());
          }
        catch (Exception e) 
          {
            r.errStream().println("Unexpected exception:");
            e.printStackTrace(r.errStream());
          }
      }
    while (true);
    

  }


  // ************************************************************
  // Trivial main()
  // ************************************************************

  public static void main(String[] argv) 
  {
    new Console("Jess Console", argv).execute();
  }
}
