/** **********************************************************************
 *  Class used to represent deffacts.
 *  A Deffacts has no ValueVector  representation; it is always a class object.
 *
 * $Id$
 * (C) 1998 E.J. Friedman-Hill and the Sandia Corporation
 ********************************************************************** */

package jess;

import java.util.*;

public class Deffacts implements Resetable
{
  
  private String m_name;
  private Vector m_facts;
  private Rete m_engine;

  String m_docstring;

  public final String name() { return m_name; }
  public final String docstring() { return m_docstring; }

  /**
    Constructor.
    */

  Deffacts(String name, Rete engine) 
  {
    m_name = name;
    m_facts = new Vector();
    m_engine = engine;
  }
  
  /**
    Add a fact to this deffacts
    */
  
  void addFact(Fact fact) throws ReteException 
  {
    addFact(fact.factData());
  }
  
  void addFact(ValueVector fact) 
  {
    m_facts.addElement(fact);
  }
  
  /**
    Assert myself into engine
    */

  public void reset() throws ReteException
  {
    for (int j=0; j<m_facts.size(); j++) 
      {
        ValueVector f = (ValueVector) m_facts.elementAt(j);
        m_engine.assert(f);
      }
  }

  /**
    Describe myself
    */
  
  public String toString() 
  {
    StringBuffer sb = new StringBuffer(100);
    sb.append("[Deffacts: ");
    sb.append(m_name);
    sb.append(" ");
    if (m_docstring != null)
      sb.append("\"" + m_docstring + "\"; ");
    sb.append(m_facts.size());
    sb.append(" facts");
    sb.append("]");
    return sb.toString();
  }

}

