/** **********************************************************************
  Some small sample user-defined math functions

  To use one of these functions from Jess, simply register the
  package class in your Java mainline:

  engine.AddUserpackage(new MathFunctions());

  $Id$
  @author Win Carus (C)1997
  ********************************************************************** */

package jess;

public class MathFunctions implements Userpackage
{
  public void add( Rete engine )
  {
    // abs added by Win Carus (9.19.97)
    engine.addUserfunction( new Abs( ) );
    // div added by Win Carus (9.19.97)
    engine.addUserfunction( new Div( ) );
    // float added by Win Carus (9.19.97)
    engine.addUserfunction( new JFloat( ) );
    // integer added by Win Carus (9.19.97)
    engine.addUserfunction( new JInteger( ) );
    // max added by Win Carus (9.19.97)
    engine.addUserfunction( new Max( ) );
    // min added by Win Carus (9.19.97)
    engine.addUserfunction( new Min( ) );
    // ** added by Win Carus (9.19.97)
    engine.addUserfunction( new Expt( ) );
    // exp added by Win Carus (9.19.97)
    engine.addUserfunction( new Exp( ) );
    // log added by Win Carus (9.19.97)
    engine.addUserfunction( new Log( ) );
    // log10 added by Win Carus (9.19.97)
    engine.addUserfunction( new Log10( ) );
    engine.addUserfunction( new Constant("pi", Math.PI ) );
    engine.addUserfunction( new Constant("e", Math.E ) );
    // round added by Win Carus (9.19.97)
    engine.addUserfunction( new Round( ) );
    // sqrt added by Win Carus (9.19.97)
    engine.addUserfunction( new Sqrt( ) );
    // random added by Win Carus (9.19.97)
    engine.addUserfunction( new JRandom( ) );
  }
}

class Abs implements Userfunction
{
  public String name( ) { return "abs" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    Value v = vv.get( 1 );
    return new Value( Math.abs( v.numericValue( ) ), v.type() );
  }
}

class Div implements Userfunction
{
  public String name( ) { return "div";}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    int first = (int)vv.get( 1 ).numericValue( );
    int second = (int)vv.get( 2 ).numericValue( );

    return new Value( ( first / second ), RU.INTEGER );
  }
}

class JFloat implements Userfunction
{
  public String name( ) { return "float" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( vv.get( 1 ).numericValue( ), RU.FLOAT );
  }
}

class JInteger implements Userfunction
{
  public String name( ) { return "integer" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( (int)vv.get( 1 ).numericValue( ), RU.INTEGER );
  }
}

class Max implements Userfunction
{
  public String name( ) { return "max" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    Value v1 = vv.get(1);
    int type = v1.type();
    double max = v1.numericValue();
    
    for (int i=2; i< vv.size(); i++)
      {
        Value v2 = vv.get(i);
        if (v2.type() == RU.FLOAT) type = RU.FLOAT;
        double n = v2.numericValue();
        if (n > max)
          max = n;
      }
    
    return new Value( max, type);
  }
}

class Min implements Userfunction
{
  public String name( ) { return "min" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    Value v1 = vv.get(1);
    int type = v1.type();
    double min = v1.numericValue();
    
    for (int i=2; i< vv.size(); i++)
      {
        Value v2 = vv.get(i);
        if (v2.type() == RU.FLOAT) type = RU.FLOAT;
        double n = v2.numericValue();
        if (n < min)
          min = n;
      }
    
    return new Value( min, type);
  }
}

class Expt implements Userfunction
{
  public String name( ) { return "**" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( Math.pow( vv.get( 1 ).numericValue( ), vv.get( 2 ).numericValue( ) ), RU.FLOAT );
  }
}

class Exp implements Userfunction
{
  public String name( ) { return "exp" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( Math.pow( Math.E, vv.get( 1 ).numericValue( ) ), RU.FLOAT );
  }
}

class Log implements Userfunction
{
  public String name( ) { return "log" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( Math.log( (double)vv.get( 1 ).numericValue( ) ), RU.FLOAT );
  }
}

class Log10 implements Userfunction
{

  private static final double log10 = Math.log( 10.0F );
  public String name( ) { return "log10" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( (Math.log( (double)vv.get( 1 ).numericValue( ) ) / log10 ), RU.FLOAT );
  }
}

class Constant implements Userfunction
{
  private Value m_val;
  private String m_name;

  Constant(String name, double value)
  {
    m_name = name; 
    try { m_val= new Value( value, RU.FLOAT ); }
    catch (ReteException re) {}
  }
  public String name() { return m_name; }

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return m_val;
  }
}
class Round implements Userfunction
{
  public String name( ) { return "round" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( Math.round( vv.get( 1 ).numericValue( ) ), RU.INTEGER );
  }
}

class Sqrt implements Userfunction
{
  public String name( ) { return "sqrt" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( Math.sqrt( vv.get( 1 ).numericValue( ) ), RU.FLOAT );
  }
}

class JRandom implements Userfunction
{
  public String name( ) { return "random" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( (int) (Math.random( ) * 65536), RU.INTEGER );
  }
}

