/** **********************************************************************
 * Conflict Resolution Strategies
 *
 * $Id$
 * (C) 1998 E.J. Friedman-Hill and the Sandia Corporation
 ********************************************************************** */
package jess;
import java.util.Vector;

public interface Strategy
{
  void addActivation(Activation a, Vector v) throws ReteException;
  String name();
}

class breadth implements Strategy
{
  public void addActivation(Activation a, Vector v) throws ReteException
  {
    int salience = a.m_rule.salience();

    // Find the first activation on the agenda with salience
    // less than this one; push it down, put this activation in its place.
    // In other words, a new activation will
    // go after any old ones of the same salience.
    for (int i=0; i<v.size(); i++)
      {
        Activation aa = (Activation) v.elementAt(i);
        if (aa.m_rule.salience() < salience)
          {
            v.insertElementAt(a, i);
            return;
          }        
      }
    v.addElement(a);
  }
  public String name() { return "breadth"; }
}

class depth implements Strategy
{
  public void addActivation(Activation a, Vector v) throws ReteException
  {
    int salience = a.m_rule.salience();

    for (int i=0; i<v.size(); i++)
      {
        Activation aa = (Activation) v.elementAt(i);
        // Find the first activation on the agenda with salience
        // less than or equal to this one; push it down, put this
        // activation in its place. In other words, a new activation will
        // go ahead of any old ones of the same salience.
        if (aa.m_rule.salience() <= salience)
          {
            v.insertElementAt(a, i);
            return;
          }        
      }
    v.addElement(a);
  }
  public String name() { return "depth"; }
}







