/** **********************************************************************
 * Example user-defined functions for the Jess Expert System Shell
 *
 * $Id$
 * (C) 1998 E.J. Friedman-Hill and the Sandia Corporation
 ********************************************************************** */

package jess;

public class StringFunctions implements Userpackage
{

  public void add( Rete engine )
  {
    engine.addUserfunction( new StrCat( ) );
    engine.addUserfunction( new StrCompare( ) );
    engine.addUserfunction( new StrIndex( ) );
    engine.addUserfunction( new SubString( ) );
    engine.addUserfunction( new StrSimple(StrSimple.LENGTH) );
    engine.addUserfunction( new StrSimple(StrSimple.UPCASE) );
    engine.addUserfunction( new StrSimple(StrSimple.LOWCASE) );
  }
}

class StrCat implements Userfunction
{
  public String name( ) { return "str-cat" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {

    if (vv.size() == 2 && vv.get(1).type() == RU.STRING)
      return vv.get(1);

    StringBuffer buf = new StringBuffer( "" );
      
    for ( int i = 1; i < vv.size( ); i++ )
      {
        Value v = vv.get(i);
        if (v.type() == RU.STRING)
          buf.append( v.stringValue());
        else
          buf.append (v.toString());
      }
      
    return new Value( buf.toString( ), RU.STRING );
      
  }
}

class StrCompare implements Userfunction
{
  public String name( ) { return "str-compare" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    return new Value( vv.get( 1 ).stringValue( ).compareTo( vv.get( 2 ).stringValue( ) ), RU.INTEGER );
  }
}

class StrIndex implements Userfunction
{
  public String name( ) { return "str-index" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    int rv = vv.get( 2 ).stringValue( ).indexOf( vv.get( 1 ).stringValue( ) );
    return rv == -1 ? Funcall.FALSE( ) : new Value( rv + 1, RU.INTEGER );
  }
}

class SubString implements Userfunction
{
  public String name( ) { return "sub-string" ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    int begin = (int) vv.get( 1 ).numericValue( ) -1;
    int end = (int) vv.get( 2 ).numericValue( );
    String s = vv.get( 3 ).stringValue( );

    if (begin < 0 || begin > s.length() - 1 ||
        end > s.length() || end <= 0)
      throw new ReteException("sub-string",
                              "Indices must be between 1 and " +
                              s.length(), "");
    return new Value(s.substring(begin, end), RU.STRING); 
  }
}

/*
  More space-efficient to only have one class here instead of three!
  */

class StrSimple implements Userfunction
{
  public static final int LENGTH=0, UPCASE=1, LOWCASE=2;
  public static final String m_names[] = {"str-length", "upcase", "lowcase"};
  
  private int m_name;
  StrSimple(int n) { m_name = n; }

  public String name( ) { return m_names[m_name] ;}

  public Value call( ValueVector vv, Context context ) throws ReteException
  {
    switch(m_name)
      {
      case UPCASE:
        return new Value(vv.get(1).stringValue().toUpperCase(), RU.STRING );
      case LOWCASE:
        return new Value(vv.get(1).stringValue().toLowerCase(), RU.STRING );
      case LENGTH:
        return new Value(vv.get(1).stringValue().length(), RU.INTEGER );
      default:
        return Funcall.NIL();
      }
  }
}
