/** **********************************************************************
 *  A tiny class to hold an individual test for a 1-input node to perform
 *  Public to work around a bug in Netscape 3.0
 * $Id$
 * (C) 1998 E.J. Friedman-Hill and the Sandia Corporation
 ********************************************************************** */

package jess;

public class Test1 implements Test
{
  /**
    What test to do (Test1.EQ, Test1.NEQ, etc)
   */

  int m_test;

  /**
    Which slot within a fact (0,1,2...)
   */

  int m_slotIdx;

  /**
    Which subslot within a multislot (0,1,2...)
   */

  int m_subIdx;

  /**
    The datum to test against
    */

  Value m_slotValue;

  /**
    A compiled form of this test
    */

  Test m_javaTest;

  /**
    Constructor
    */
  Test1(int test, int slot_idx, Value slot_value)
       throws ReteException
  {
    this(test, slot_idx, -1, slot_value, null);
  }

  Test1(int test, int slot_idx, int sub_idx, Value slot_value, Test javaTest) 
       throws ReteException
  {
    m_test = test;
    m_slotIdx = slot_idx;
    m_subIdx = sub_idx;
    m_slotValue = slot_value;
    m_javaTest = javaTest;
  }

  public boolean doTest(Token lt, ValueVector rf, EvalCache cache, Rete engine)
       throws ReteException
  {
    boolean retval;
    
    if (m_javaTest != null)
      retval = m_javaTest.doTest(lt, rf, cache, engine);
    else
      retval = Funcall.execute(eval(m_slotValue, lt, rf, cache),
                               engine.globalContext(),
                               cache).equals(Funcall.FALSE());
    switch (m_test) 
      {
      case EQ:
        if (retval)
          return false;
        break;
        
      case NEQ:
        if (!retval)
          return false;
        break;
        
      }
    return true;
  }

  final Funcall eval(Value v, Token t, ValueVector f, EvalCache cache)
       throws ReteException
  {
    Funcall vv = (Funcall) v.funcallValue();
    int size = vv.size();

    if (size == 1)
      return vv;
    else
      vv = vv.cloneInto(cache.getFuncall());
    
    for (int i=1; i<size; i++)
      {
        Value value = vv.get(i);
        int type = value.type();

        switch (type)
          {
          case RU.INTARRAY:
            {
              int[] binding = value.intArrayValue();
              Value var;
              if (binding[0] == t.size())
                var = f.get(binding[1]);
              else
                var = t.fact(binding[0]).get(binding[1]);

              if (binding[2] == -1)
                vv.set(var, i);
              else
                {
                  ValueVector subv = var.listValue();
                  vv.set(subv.get(binding[2]), i);
                }
              break;
            }
          case RU.FUNCALL:
            {
              Value val = cache.getValue();
              val.resetValue(eval(value, t, f, cache), RU.FUNCALL);
              vv.set(val, i);
              break;
            }
          
          }
      }
    return vv;
  }

  public String toString()
  {
    StringBuffer sb = new StringBuffer(100);
    sb.append("[Test1: test=");
    sb.append(m_test == NEQ ? "NEQ" : "EQ");
    sb.append(";slot_idx=");
    sb.append(m_slotIdx);
    sb.append(";sub_idx=");
    sb.append(m_subIdx);
    sb.append(";slot_value=");
    sb.append(m_slotValue);
    sb.append("]");

    return sb.toString();
  } 


}



