/*
 * URLDatabase.java
 *	Providing Database interface to WebRobot for storing URLs
 *	using Sleepycat's DB (so-called Berkeley DB)
 *
 * This program is designed for CS571 Computer Graphics of POSTECH
 *
 * Copyright (c) Spring 1998, Dept of Computer Science & Engineering
 * Namgoong Jae-chang (sinclair@postech.ac.kr)
 *
 * $Id: URLDatabase.java,v 1.3 1998/06/22 09:06:54 sinclair Exp $
 */

import com.sleepycat.db.*;

class URLDatabase extends DbEnv
{
    private static final String FileName = "url.db";

    public URLDatabase(String home) throws DbException
    {
        super(home, null, 0);

        try {
            set_error_stream(System.err);
            set_errpfx("URLDatabase");
        }
        catch (Exception e) {
            System.err.println(e.toString());
            System.exit(1);
        }
    }

    public void save(String url) throws DbException
    {
	Db table = Db.open(FileName, Db.DB_BTREE, Db.DB_CREATE,
		0644, this, null);

	Dbt key = new Dbt(url.getBytes());
	key.set_size(url.length());
	Dbt data = new Dbt("0".getBytes());
	data.set_size(1);

	try {
	    int err;
	    if ((err = table.put(null, key, data, Db.DB_NOOVERWRITE)) == Db.DB_KEYEXIST) {
		System.out.println(" Above URL already exists!");
	    }
	}
	catch (DbException dbe) {
	    System.err.println(dbe.toString());
	}

	table.close(0);
    }

    public String load() throws DbException
    {
	String rtn = null;
	Db table = Db.open(FileName, Db.DB_BTREE, Db.DB_CREATE,
		0644, this, null);

	// Acquire an iterator for the table.
	Dbc iterator;
	iterator = table.cursor(null);

	Dbt key = new Dbt();
	key.set_flags(Db.DB_DBT_MALLOC);
	Dbt data = new Dbt();
	data.set_flags(Db.DB_DBT_MALLOC);

	while (iterator.get(key, data, Db.DB_NEXT) == 0) {
	    String key_string = new String(key.get_data(), 0, key.get_size());
	    String data_string = new String(data.get_data(), 0, data.get_size());

	    if (data_string.equals("0") == true) {
		rtn = key_string;
		break;
	    }
	}

	iterator.close();
	table.close(0);
	return (rtn);
    }

    public void check(String url) throws DbException
    {
	Db table = Db.open(FileName, Db.DB_BTREE, Db.DB_CREATE,
		0644, this, null);

	// Acquire an iterator for the table.
	Dbc iterator;
	iterator = table.cursor(null);

	Dbt key = new Dbt();
	key.set_flags(Db.DB_DBT_MALLOC);
	Dbt data = new Dbt();
	data.set_flags(Db.DB_DBT_MALLOC);

	while (iterator.get(key, data, Db.DB_NEXT) == 0) {
	    String key_string = new String(key.get_data(), 0, key.get_size());

	    if (key_string.equals(url) == true) {
		data.set_data("1".getBytes());
		data.set_size(1);
		iterator.put(key, data, Db.DB_CURRENT);
		break;
	    }
	}

	iterator.close();
	table.close(0);
    }

    public void print() throws DbException
    {
	Db table = Db.open(FileName, Db.DB_BTREE, Db.DB_CREATE,
		0644, this, null);

	// Acquire an iterator for the table.
	Dbc iterator;
	iterator = table.cursor(null);

	Dbt key = new Dbt();
	key.set_flags(Db.DB_DBT_MALLOC);
	Dbt data = new Dbt();
	data.set_flags(Db.DB_DBT_MALLOC);

	while (iterator.get(key, data, Db.DB_NEXT) == 0) {
	    String key_string = new String(key.get_data(), 0, key.get_size());
	    String data_string = new String(data.get_data(), 0, data.get_size());

	    System.out.println("URL: " + key_string);
	    System.out.println("Visited: " + data_string);
	}

	iterator.close();
	table.close(0);
    }

    public void removeInnerNode() throws DbException
    {
	Db table = Db.open(FileName, Db.DB_BTREE, Db.DB_CREATE,
		0644, this, null);

	// Acquire an iterator for the table.
	Dbc iterator;
	iterator = table.cursor(null);

	Dbt key = new Dbt();
	key.set_flags(Db.DB_DBT_MALLOC);
	Dbt data = new Dbt();
	data.set_flags(Db.DB_DBT_MALLOC);

	while (iterator.get(key, data, Db.DB_NEXT) == 0) {
	    String key_string = new String(key.get_data(), 0, key.get_size());

	    if (key_string.toUpperCase().indexOf("TITLE.CFM") == -1) {
		data.set_data("1".getBytes());
		data.set_size(1);
		iterator.put(key, data, Db.DB_CURRENT);
	    }
	}

	iterator.close();
	table.close(0);
    }

    public void regainLeafNode() throws DbException
    {
	Db table = Db.open(FileName, Db.DB_BTREE, Db.DB_CREATE,
		0644, this, null);

	// Acquire an iterator for the table.
	Dbc iterator;
	iterator = table.cursor(null);

	Dbt key = new Dbt();
	key.set_flags(Db.DB_DBT_MALLOC);
	Dbt data = new Dbt();
	data.set_flags(Db.DB_DBT_MALLOC);

	while (iterator.get(key, data, Db.DB_NEXT) == 0) {
	    String key_string = new String(key.get_data(), 0, key.get_size());

	    if (key_string.toUpperCase().indexOf("TITLE.CFM") != -1) {
		data.set_data("0".getBytes());
		data.set_size(1);
		iterator.put(key, data, Db.DB_CURRENT);
	    }
	}

	iterator.close();
	table.close(0);
    }
}
